using Colors
using DelimitedFiles
using FileIO
using ImageIO
using Statistics

path = joinpath(@__DIR__, "..", "fig1_rotated_noticks_nohline_framed_trimmed.png")
pic = load(path)
T = eltype(pic)
@assert T <: GrayA "Need a picture in gray scale mode"

iswhite(c) = c.val >= 0.8
isblack(c) = c.val <= 0.6

# find pixel boundaries of frame
h, w = size(pic)

px_xmins_outer = Tuple{Int,Int}[]
px_xmins_inner = Tuple{Int,Int}[]
for (i,row) in enumerate(eachrow(pic))
    idx = findfirst(isblack, row)
    isnothing(idx) && continue
    push!(px_xmins_outer, (i,idx))
    idx_inner = findfirst(iswhite, view(row,idx:w))
    isnothing(idx_inner) && continue
    push!(px_xmins_inner, (i,idx_inner-2+idx)) # -2, because we want the last not white pixel
end

px_xmaxs_outer = Tuple{Int,Int}[]
px_xmaxs_inner = Tuple{Int,Int}[]
for (i,row) in enumerate(eachrow(pic))
    idx = findfirst(isblack, reverse(row))
    isnothing(idx) && continue
    push!(px_xmaxs_outer, (i,w+1-idx))
    idx_inner = findfirst(iswhite, reverse(view(row,1:w+1-idx)))
    isnothing(idx_inner) && continue
    push!(px_xmaxs_inner, (i,w+2-idx-idx_inner)) # +2, because we ant the last not white pixel
end

px_ymins_outer = Tuple{Int,Int}[]
px_ymins_inner = Tuple{Int,Int}[]
for (i,col) in enumerate(eachcol(pic))
    idx = findfirst(isblack, col)
    isnothing(idx) && continue
    push!(px_ymins_outer, (idx,i))
    idx_inner = findfirst(iswhite, view(col,idx:h))
    isnothing(idx_inner) && continue
    push!(px_ymins_inner, (idx_inner-2+idx,i)) # -2, because we ant the last not white pixel
end

px_ymaxs_outer = Tuple{Int,Int}[]
px_ymaxs_inner = Tuple{Int,Int}[]
for (i,col) in enumerate(eachcol(pic))
    idx = findfirst(isblack, reverse(col))
    isnothing(idx) && continue
    push!(px_ymaxs_outer, (h+1-idx,i))
    idx_inner = findfirst(iswhite, reverse(view(col,1:h+1-idx)))
    isnothing(idx_inner) && continue
    push!(px_ymaxs_inner, (h+2-idx-idx_inner,i)) # +2, because we ant the last not white pixel
end


# frame should be drawn exactly horizontally and vertically
@assert length(px_xmins_outer) == length(px_xmaxs_outer)
@assert length(px_ymins_outer) == length(px_ymaxs_outer)

# compute pixel range for inside of frame
xmins_outer = [ px[2] for px in px_xmins_outer ]
xmaxs_outer = [ px[2] for px in px_xmaxs_outer ]
ymins_outer = [ px[1] for px in px_ymins_outer ]
ymaxs_outer = [ px[1] for px in px_ymaxs_outer ]

xmin_outer = round(Int, mean(xmins_outer))
xmax_outer = round(Int, mean(xmaxs_outer))
ymin_outer = round(Int, mean(ymins_outer))
ymax_outer = round(Int, mean(ymaxs_outer))

# compute pixel range for inside of frame
xmins_inner = [ px[2] for px in px_xmins_inner ]
xmaxs_inner = [ px[2] for px in px_xmaxs_inner ]
ymins_inner = [ px[1] for px in px_ymins_inner ]
ymaxs_inner = [ px[1] for px in px_ymaxs_inner ]

xmin_inner = round(Int, mean(xmins_inner))
xmax_inner = round(Int, mean(xmaxs_inner))
ymin_inner = round(Int, mean(ymins_inner))
ymax_inner = round(Int, mean(ymaxs_inner))

# frame width at each end
dxmin = xmin_inner - xmin_outer
dxmax = xmax_outer - xmax_inner
dymin = ymin_inner - ymin_outer
dymax = ymax_outer - ymax_inner

# skip a few pixels to the left and right when searching for the graph in the frame,
# because we manually erased some ticks which might have caused artifacts
# also the plot is rather inaccurate around the left axis
dx, dy = 2, 2

# graph
xs    = xmin_inner+dx:xmax_inner-dx
graph = view(pic, ymin_inner+dy:ymax_inner-dy, xmin_inner+dx:xmax_inner-dx)

# find envelope pixels of the curve
ys_min = zeros(length(xs))
ys_max = zeros(length(xs))
for (i,col) in enumerate(eachcol(graph))
    idx_black = findfirst(isblack, col)
    isnothing(idx_black) && error("Something went wrong when finding the lower envelop pixel!")
    ys_min[i] = idx_black - 1
    idx_white = findfirst(iswhite, view(col, idx_black:length(col)))
    isnothing(idx_white) && error("Something went wrong when finding the upper envelop pixel!")
    ys_max[i] = idx_black + idx_white - 2 # -2, because we want the last not white pixel
end

# undo pixel offsets
ys_min .+= ymin_inner .+ dy
ys_max .+= ymin_inner .+ dy

# take the mean as the true value
ys = [ round(Int, (ymax+ymin)/2) for (ymin,ymax) in zip(ys_min,ys_max) ]

# convert pixels in coordinates
coord_xmin, coord_xmax = 0.0, 10.0
coord_ymin, coord_ymax = -0.12, 0.12

Lx = coord_xmax-coord_xmin
Ly = coord_ymax-coord_ymin

# we take the coordinate limits to be at the center of the frame boundaries
xmin = round(Int, (xmin_outer+xmin_inner)/2)
xmax = round(Int, (xmax_outer+xmax_inner)/2)
ymin = round(Int, (ymin_outer+ymin_inner)/2)
ymax = round(Int, (ymax_outer+ymax_inner)/2)
# number of pixels inside plot
Nx = xmax-xmin+1
Ny = ymax-ymin+1
# length per pixel
dLx, dLy = Lx/(Nx-1), Ly/(Ny-1)

# graph in coordinates
# remove half of the frame width so that coordinates are centered
# remove half a pixel for pixel center
offset_x = round(Int, dxmin/2) + 0.5
# I messed up somewhere, need to add a factor 2 to fix that
offset_y = round(Int, dymin/2) + 0.5 + 2
coords_xs     = @. dLx * (xs - offset_x) + coord_xmin
coords_ys_min = @. dLy * (ys_min - offset_y) + coord_ymin
coords_ys_max = @. dLy * (ys_max - offset_y) + coord_ymin
coords_ys     = @. dLy * (ys - offset_y) + coord_ymin

# coordinate width of frame
coord_dxmin = dLx * dxmin
coord_dxmax = dLx * dxmax
coord_dymin = dLy * dymin
coord_dymax = dLy * dymax

# save data
open(joinpath(@__DIR__, "data.txt"), "w") do io
    println(io, "# frame widths : $coord_dxmin, $coord_dxmax, $coord_dymin, $coord_dymax")
    writedlm(io, zip(coords_xs, coords_ys, coords_ys_min, coords_ys_max))
end
