using DelimitedFiles
using LsqFit
using Plots

path = joinpath(@__DIR__, "..", "fig1_rotated_noticks_nohline_framed_trimmed.png")
pic = load(path)
coord_xmin, coord_xmax = 0.0, 10.0
coord_ymin, coord_ymax = -0.12, 0.12

frame_widths, data = open(joinpath(@__DIR__, "data.txt"), "r") do file
    line = readline(file)
    m = match(r"# frame widths : (.*)", line)
    isnothing(m) && error("Failed to extract frame width data")
    widths = split(m[1], ',')
    widths = tryparse.(Float64, widths)
    data = readdlm(joinpath(@__DIR__, "data.txt"), skipstart=1)
    frame, data
end

w_xmin, w_xmax, w_ymin, w_ymax = frame_widths

xs     = view(data, :, 1)
ys     = view(data, :, 2)
ys_min = view(data, :, 3)
ys_max = view(data, :, 4)

ny, nx = size(data)
ratio = ny/nx

plt = Plots.plot(;
                 aspect_ratio=:none, legend=:bottomright,
                 # aspect_ratio=ratio, legend=:outerbottomright,
                 xlims=(-coord_xmax,coord_xmax),
                 ylims=(-coord_ymax,coord_ymax))
Plots.plot!([coord_xmin-w_xmin/2,coord_xmax+w_xmax/2],
            [coord_ymin-w_ymin/2,coord_ymax+w_ymax/2],
            pic)
Plots.plot!(xs, ys, label="ys")
# Plots.plot!(xs, ys_min, label="ys_min")
# Plots.plot!(xs, ys_max, label="ys_max")


# @. model(x, p) = p[1] * (x^p[2] - p[3] * x^p[4]) * exp(-x^p[5] / p[6]) + p[7]
# guess = [ -0.3, 1.0, 1.0, 3.0, 2.0, 1.0, 0.0 ]

# fit = curve_fit(model, xs, ys, guess)
# Plots.plot!(xs, model.(xs, Ref(guess)), label="model(guess)")

# mirror data for fit to impose odd parity
# @. model(x, p) = (p[1] * x + p[2] * x^5) * exp(-x^2 / p[4]) + p[3]

@. model(x, p) = (p[1] * x + p[2] * x^5) * exp(-x^2 / p[4]) + p[3]
guess = [ -0.25, 0.1, 0.0, 1.0]

# @. model(x, p) = (p[1] * x + p[2] * x^3) * exp(-x^2) + p[3]
# guess = [ -1.0, 1.0, 0.0 ]

xxs = vcat(-reverse(xs), xs)
yys = vcat(-reverse(ys), ys)
fit = curve_fit(model, xxs, yys, guess)

Plots.plot!(xxs, yys, label="mirrored ys")
# Plots.plot!(xxs, model.(xxs, Ref(guess)), label="model(guess)")
Plots.plot!(xxs, model.(xxs, Ref(coef(fit))), label="model(fit)")
Plots.plot!(xxs, abs.(yys .- model.(xxs, Ref(coef(fit)))), label="|yys - model(fit)|")

display(plt)

println("best coeffs")
coeffs = coef(fit)
display(coeffs)

println("coeffs[1]/coeffs[2] = $(coeffs[1]/coeffs[2])")

